install.packages(c('tidyverse', 'doParallel', 'doRNG')) # only needs to be run once, then can be commented out

library('tidyverse')
library('doParallel')
library('doRNG')

registerDoParallel(cores=4) # can be adjusted based on your machine
set.seed(8675309)

##################################################################################
# function to simulate oscillations

simulateFromParam = function(param, nIter) {
	switchFrac = runif(nIter)

	extrema = with(param, {
		foreach(ii=1:nIter, .combine=rbind) %dorng% {
			interTimesPre = periodPre + rnorm(nCycles, sd=sdRef)
			extremaPre = tibble(epoch = 'pre', timeDiff = c(0, interTimesPre)) %>%
				mutate(time = cumsum(timeDiff))

			tSwitch = max(extremaPre$time) + switchFrac[ii] * periodPre
			tStartPost = tSwitch + (1 - switchFrac[ii] + (-phaseShiftFrac %% 1)) * periodPost

			interTimesPost = periodPost + rnorm(nCycles, sd=sdRef)
			extremaPost = tibble(epoch = 'post', timeDiff = c(0, interTimesPost)) %>%
				mutate(time = tStartPost + cumsum(timeDiff))

			extremaNow = rbind(extremaPre, extremaPost) %>%
				mutate(simIdx = ii, time = time - tSwitch) %>%
				select(-timeDiff)}})}

##################################################################################
# generate simulations
# these will be slightly different from those used in the paper, which were generated
# before switching the parallel execution to %dorng% to make them exactly reproducible

nIter = 100
params = expand.grid(nCycles = 3:4,
							 sdRef = c(0.5, 0.25),
							 periodPre = 24,
							 periodPost = c(23, 24, 26),
							 phaseShiftFrac = c(-0.05, 0, 0.1))
params$paramIdx = 1:nrow(params)

extremaAll = foreach(ii=1:nrow(params), .combine=rbind) %dorng% {
	simulateFromParam(params[ii,], nIter) %>%
		mutate(paramIdx = ii)}

write_csv(extremaAll, 'tipa_sim_extrema.csv')
write_csv(params, 'tipa_sim_params.csv')
